/**
*	@author: Thomas Brandého
*/

#ifndef INPUT_H
#define INPUT_H
#include "keybindings.h"
#include <queue>
#include <vector>
#include <unordered_map>
#include "textbuffer.h"

class Input{
	public:
		/* Constructors */
		Input(sf::Window *w);

		/* general action-mapping functions */
        void setKeysMap(IKeysMap km);  //set bindings
        void updateEvents();           //handle the input and put the associated event in the file
        std::vector<int> getActions();               //get the first action in the file of event

		/* context-related functions */
		void addContext(Context context);
		void setCurrentContext(std::string context_name);
		void updateKeyBindings();

		/* window-related function */
        bool isCloseRequested() const;
        bool isResized() const;

		/* keyboard-related functions */
		bool isKeyPressed(int key) const;

		/* mouse-related function */
		sf::Vector2i getPosition() const;
		sf::Vector2i getDeltaPosition() const;
		float getDeltaVerticalScroll() const;

		/* text-related function */
		std::string getText();

		void test();

	private:
		/* window-related variables */
        sf::Window* m_window;
        bool m_closeRequested;
        bool m_hasBeenResized;

		/* general action-mapping variables */
        IKeysMap m_keysmap;
        std::vector<int> m_actions;
		int nb_actions;

		/* context-related variables */
        std::string m_current_context;
		//std::unordered_map<std::string, std::vector<int>> contexts;
        std::vector<Context> m_contexts;
        std::unordered_map<std::string,KeyBindings> m_keybindings;

		/* keyboard-related variables */
        std::vector<sf::Keyboard::Key> m_heldkeys;
		void releaseHeldKeys(sf::Keyboard::Key keycode);

		/* mouse-related variables */
        sf::Vector2i m_mouse_position;
        sf::Vector2i m_last_mouse_position;
        float m_delta_vertical_scroll;

		/* text-related variable */
        std::string m_buffer;
};

#endif //INPUT_H
